#include "jukebox.h"

#define BUFSIZE 1024
unsigned char buf[BUFSIZE];

#define RADIO_DEVICE "/dev/radio"
#define PRESETFILE "radio.presets"

#define RADIO_VOLUME 0x1000

int radio_fd;

int radio_freq = 999;   // 99.9MHz default

struct {
   char *name;
   int freq;
} radio_presets[100];

int n_radio_presets = 0;

set_radio_preset(int n)
{
   if(n<n_radio_presets) {
       radio_freq = radio_presets[n].freq;
       return radio_freq;
   }
   return 0;
}

read_radio_presets()
{
   FILE *fp = fopen(PRESETFILE,"r");
   if(!fp) {
      printf("Could not read %s\n",PRESETFILE);
      return;
   }
   while(fgets(buf,sizeof(buf),fp)) {
      int p,state;
      if(*buf == '#') continue;
      while(p = strlen(buf)) {
         if(isspace(buf[p-1])) buf[p-1] = 0; else break;
      }
      state = 0;
      p = 0;
      do {
         char c = buf[p++];
         if(!c) break;
         switch(state) {
            case 0:
               if(!index("0123456789.",c)) {
                  if(isspace(c)) state = 1; else  state = 3;
               }
               break;
            case 1:
               if(!isspace(c)) {
                  p--;
                  state = 2;
               }
         }
      } while(state<2);
      if(state!=2) {
         fprintf(stderr,"%s parse error: %s\n",PRESETFILE,buf);
      } else {
         radio_presets[n_radio_presets].name = strdup(buf+p);
         radio_presets[n_radio_presets].freq = (int) 10.0*atof(buf)+0.5;
         printf("radio preset: %d = \"%s\"\n",radio_presets[n_radio_presets].freq,
            radio_presets[n_radio_presets].name);
         n_radio_presets++;
      }
   }
   fclose(fp);
}

radio_init()
{
   radio_fd = open(RADIO_DEVICE,O_RDONLY);
   if(radio_fd<0) {
      fprintf(stderr,"%s not available\n",RADIO_DEVICE);
      return;
   }
   radio_volume(0);
   read_radio_presets();
   if(n_radio_presets) radio_freq = radio_presets[0].freq;
   radio_tune(radio_freq);
}

//  radio_tune: freq = 100KHz multiple, e.g. 106.1MHz = 1061

radio_tune(int freq)
{
   unsigned long f = 1600*freq;
   if(radio_fd<0) return;
   //printf("radio_tune: %d\n",freq);
   ioctl(radio_fd,VIDIOCSFREQ,&f);
}

//  radio_volume: vol = 0   hardware mute
//                vol = n   1..65535 volume setting

radio_volume(int vol)
{
   struct video_audio audio;
   if(radio_fd<0) return;
   //printf("radio_volume: %d\n",vol);
   ioctl(radio_fd,VIDIOCGAUDIO,&audio);
   audio.mode = VIDEO_SOUND_STEREO;
   audio.volume = vol;
   if(vol) {
      audio.flags &= ~VIDEO_AUDIO_VOLUME;
   } else {
      audio.flags |= VIDEO_AUDIO_MUTE;
   }
   ioctl(radio_fd,VIDIOCSAUDIO,&audio);
}

radio_mute()
{
   radio_volume(0);
}

radio_unmute()
{
   radio_volume(RADIO_VOLUME);
}

radio_tune_down()
{
   radio_freq = (radio_freq & 1) ? radio_freq-2 : radio_freq-1;
   if(radio_freq < 881) radio_freq = 1081;
}

radio_tune_up()
{
   radio_freq = (radio_freq & 1) ? radio_freq+2 : radio_freq+1;
   if(radio_freq > 1081) radio_freq = 881;
}

char *presetname(int freq)
{
   int i;
   for(i=0;i<n_radio_presets;i++) {
      if(radio_presets[i].freq == radio_freq) return radio_presets[i].name;
   }
   return 0;
}

find_next_radio_preset(int dir)
{
    int found_preset = 0;
    int f;
    radio_freq |= 1;  // stations are at odd multiples of 100KHz
    f = radio_freq;
    do {
       int i;
       if(dir<0) radio_tune_down(); else radio_tune_up();
       if(presetname(radio_freq)) break;
    } while(f != radio_freq);
}
