#include "jukebox.h"

#define REC_DSP "/dev/dsp"

#define REC_FRAGSIZE_LOG2 12
#define REC_NUMFRAGS 20

int rec_ctrl_fd[2];
int rec_status_fd[2];

//------------------------------------------------------------------------------
//  Record Process
//
//  Forked off the main program when radio mode is selected
//
//  Listens for characters on rec_ctrl_fd[0]
//
//     REC_CTRL_START    - start recording
//     REC_CTRL_STOP     - stop recording and save fragment
//     REC_CTRL_QUIT     - quit
//

int rec_fd = -1;

char recbuf[4096];
char rec_filename[1000];
char temp_filename[1000];

record_main()
{
   fd_set readfds;
   int sel_result;
   int recording = 0;
   int maxfd;
   int p;
   int i;
   int bytecount;
   int secs_space;
   char c;
   int lsecs;
   FILE *recfile_fp;
   int raw_flag;

   //printf("record process started\n");

   for(;;) {
      FD_ZERO(&readfds);
      FD_SET(rec_ctrl_fd[0],&readfds);
      maxfd = rec_ctrl_fd[0];
      if(rec_fd >= 0) {
         FD_SET(rec_fd,&readfds);
         if(rec_fd > maxfd) maxfd = rec_fd;
      }
      do {
         sel_result=select(maxfd+1,&readfds,0,0,0);
      } while(sel_result<0 && errno==EINTR);
      if(sel_result<0) {
         perror("select");
         exit(999);
      }

      if(rec_fd >= 0 && FD_ISSET(rec_fd,&readfds)) {
         int frag = read(rec_fd,recbuf+p,sizeof(recbuf)-p);
         int secs;
         p += frag;
         if(p==sizeof(recbuf)) {
            if(p != fwrite(&recbuf,1,p,recfile_fp)) {
                signal_disk_full();
                fclose(recfile_fp);
                close(rec_fd);
                rec_fd = -1;
                recording = 0;
            }
            bytecount += p;
            p=0;
         }
         secs = secs_space - (bytecount+88199) / 88200;
         if(secs != lsecs) {
            unsigned char buf[3];
            buf[0] = REC_STATUS_RECORDING;
            buf[1] = secs;
            buf[2] = secs >> 8;
            write(rec_status_fd[1],&buf,3);
         }
         lsecs = secs;
      }

      if(FD_ISSET(rec_ctrl_fd[0],&readfds)) {
         read(rec_ctrl_fd[0],&c,1);
         //printf("record process got %d\n",c);
         switch(c) {
            case REC_CTRL_START:
               if(!recording) {
                  time_t starttime;
                  struct tm *st_tm;
                  unsigned long bytes_space = 1024 * get_space(RECFILE_PATH);
                  secs_space = bytes_space/88200;
                  printf("%d bytes free = %d seconds\n",bytes_space,secs_space);
                  if(secs_space<10) {
                     signal_disk_full();
                     break;
                  }

                  starttime = time(0);
                  st_tm = localtime(&starttime);

                  sprintf(rec_filename,"%s/record-%04d%02d%02d-%02d%02d%02d",
                     RECFILE_PATH,st_tm->tm_year+1900,st_tm->tm_mon+1,st_tm->tm_mday,
                     st_tm->tm_hour,st_tm->tm_min,st_tm->tm_sec);
                  sprintf(temp_filename,"%s/record.temp",RECFILE_PATH);

                  recfile_fp = fopen(temp_filename,"w");
                  if(!recfile_fp) {
                     signal_disk_full();  // any open error will say out of space
                     break;
                  }

                  // Ugly brute-force hack.  When killing mpg123 it can take a while
                  // before closing the sound driver.  Until this happens, this open 
                  // won't succeed.  The following loop keeps trying for 2 seconds.

                  for(i=0;i<15;i++) { 
                     open_rec_audio();
                     if(rec_fd>=0) break;
                     usleep(200000);
                  }

                  if(rec_fd<0) break;  // better error handling later

                  bytecount = 0;
                  lsecs = -1;
                  recording = 1;
                  raw_flag = 0;

                  // The following seems to kick the audio driver
                  // into starting to record (otherwise select()
                  // will never show ready data).

                  p = read(rec_fd,recbuf,4);
               }
               break;
            case REC_CTRL_STOP:
               if(recording) {
                  fclose(recfile_fp);
                  close(rec_fd);
                  if(raw_flag) {
                     strcat(rec_filename,".raw");
                  }
                  rename(temp_filename,rec_filename);                  
                  rec_fd = -1;
               }
               recording = 0;
               c = REC_STATUS_STOPPED;
               write(rec_status_fd[1],&c,1);
               break;
            case REC_CTRL_DONT_COMPRESS:
               if(recording) {
                  raw_flag = 1;
                  c = REC_STATUS_OK_NO_COMPRESS;
                  write(rec_status_fd[1],&c,1);
               }
               break;
            case REC_CTRL_QUIT:
               if(recording) fclose(recfile_fp);
               //printf("record process exiting\n");
               exit(0);
         }
      }
   }
}

signal_disk_full()
{
   char c = REC_STATUS_DISK_FULL;
   write(rec_status_fd[1],&c,1);
}

int get_space(char *path)
{
   char cmdbuf[1000];
   int nkb = 0;
   int c;
   FILE *fp;
   sprintf(cmdbuf,"df -k %s | tail -1 | awk '{print $4}'",path);
   fp = popen(cmdbuf,"r");
   while((c=getc(fp))!=EOF) if(isdigit(c)) nkb = 10*nkb+c-'0';
   fclose(fp);
   printf("get_space returns %d\n",nkb);
   return nkb;
}

open_rec_audio()
{
   int r;
   unsigned val;

   int mixfd = open("/dev/mixer",0);
   if(mixfd<0) {
      perror("mixer open");
      exit(999);
   }
   val = 1 << SOUND_MIXER_LINE;
   if(0>ioctl(mixfd,SOUND_MIXER_WRITE_RECSRC,&val)) {
      perror("mixer record source select");
      exit(999);
   }
   close(mixfd);

   rec_fd = open(REC_DSP,O_RDONLY);
   if(rec_fd<0) {
      fprintf(stderr,"can't open audio driver for record\n");
      return;
   }
   r = ioctl(rec_fd,SNDCTL_DSP_RESET,NULL);
   if(r>=0) {
      // Do not check the return code from the fragment size set...
      // The SB16 driver returns an error if it can't set up the
      // exact parameters requested.  However it still seems to do
      // its best.

      val = (REC_NUMFRAGS<<16) | REC_FRAGSIZE_LOG2;
      ioctl(rec_fd,SNDCTL_DSP_SETFRAGMENT,&val);
   }
   if(r>=0) {
      val = 16;
      r = ioctl(rec_fd,SNDCTL_DSP_SAMPLESIZE,&val);
      if(r<0) printf("sound driver rejected SAMPLESIZE\n");
   }
   if(r>=0) {
      val = AFMT_S16_LE;
      r = ioctl(rec_fd,SNDCTL_DSP_SETFMT,&val);
      if(r<0) printf("sound driver rejected SETFMT\n");
   }
   if(r>=0) {
      val = 1;
      r = ioctl(rec_fd,SNDCTL_DSP_STEREO,&val);
      if(r<0) printf("sound driver rejected STEREO\n");
   }
   if(r>=0) {
      val = 22050;
      r = ioctl(rec_fd,SNDCTL_DSP_SPEED,&val);
      if(r<0) printf("sound driver rejected SPEED\n");
   }
   if(r>=0) {
      audio_buf_info info;
      r = ioctl(rec_fd,SNDCTL_DSP_GETISPACE,&info);
      if(r<0) printf("sound driver rejected GETISPACE\n");
      printf("rec buffer info: %d fragments of %d bytes\n",info.fragstotal,info.fragsize);
   }
   if(r<0) {
      fprintf(stderr,"record sound device did not accept settings.\n");
      close(rec_fd);
      rec_fd = -1;
   }
}
