#include "jukebox.h"
#include "buttons.h"

struct kmap {
   char c;
   int code;
};

struct kmap play_key_mappings[] = {
   {'[',VOL_DOWN_CODE},
   {']',VOL_UP_CODE},
   {'0',ZERO_CODE},
   {'1',ONE_CODE},
   {'2',TWO_CODE},
   {'3',THREE_CODE},
   {'4',FOUR_CODE},
   {'5',FIVE_CODE},
   {'6',SIX_CODE},
   {'7',SEVEN_CODE},
   {'8',EIGHT_CODE},
   {'9',NINE_CODE},
   {'*',STAR_CODE},
   {'M',MUTE_CODE},
   {'F',FAVE_TOGGLE_CODE},
   {'+',NEXT_FAVE_CODE},
   {'=',NEXT_FAVE_CODE},
   {'-',PREV_FAVE_CODE},
   {'T',TIME_CODE},
   {'P',PROFILE_CODE},
   {'/',FIND_CODE},
   {'C',CUCKOO_CODE},
   {'A',ALBUM_TOGGLE_CODE},
   {8,STOP_CODE},
   {127,STOP_CODE},
   {13,PLAY_CODE},
   {'<',HIST_BACKWARD_CODE},
   {'>',HIST_FORWARD_CODE},
   {'N',RANDOM_CODE},
   {'R',RADIO_CODE},
   {'#',POUND_CODE},
   {0,0}
};

struct kmap radio_key_mappings[] = {
   {'[',VOL_DOWN_CODE},
   {']',VOL_UP_CODE},
   {'0',ZERO_CODE},
   {'1',ONE_CODE},
   {'2',TWO_CODE},
   {'3',THREE_CODE},
   {'4',FOUR_CODE},
   {'5',FIVE_CODE},
   {'6',SIX_CODE},
   {'7',SEVEN_CODE},
   {'8',EIGHT_CODE},
   {'9',NINE_CODE},
   {'*',STAR_CODE},
   {'M',MUTE_CODE},
   {'F',FAVE_TOGGLE_CODE},
   {'+',NEXT_FAVE_CODE},
   {'=',NEXT_FAVE_CODE},
   {'-',PREV_FAVE_CODE},
   {'T',TIME_CODE},
   {'/',FIND_CODE},
   {'C',CUCKOO_CODE},
   {'A',ALBUM_TOGGLE_CODE},
   {8,STOP_CODE},
   {127,STOP_CODE},
   {13,PLAY_CODE},
   {'<',HIST_BACKWARD_CODE},
   {'>',HIST_FORWARD_CODE},
   {'N',RANDOM_CODE},
   {'R',ALBUM_TOGGLE_CODE},
   {'#',POUND_CODE},
   {0,0}
};

struct kmap record_key_mappings[] = {
   {'[',VOL_DOWN_CODE},
   {']',VOL_UP_CODE},
   {'M',MUTE_CODE},
   {'R',ALBUM_TOGGLE_CODE},
   {'W',PROFILE_CODE},
   {0,0}
};

char common_remotedisplaystr[] = 
"\033[H\033[2J"
"+------------------------------------------+\r\n"
"|                                          |\r\n"
"|                                          |\r\n"
"+------------------------------------------+\r\n"
"\r\n";

char play_remotedisplaystr[] = 
"CR    = Play/Pause      BS   = Stop/Play Mode\r\n"
"Right = Next Track      Left = Previous Track\r\n"
"Down  = Next Album      Up   = Previous Album\r\n"
"[]    = Volume          M    = Mute\r\n"
"+-    = Next/Prev Fave  F    = Toggle Fave\r\n"
"A     = Album/Artist    /    = Find Track\r\n"
"<     = History Back    >    = History Forward\r\n"
"P     = Profile Select  N    = Random Track\r\n"
"\r\n"
"0-9   = Go to 0%, 10%, 20%, ... (if possible)\r\n"
"\r\n"
"T     = Talking Clock   C    = Cuckoo Mode\r\n"
"R     = Radio Mode      *    = Radio Recordings\r\n"
"\r\n"
"^L    = Refresh display (all modes)\r\n"
;

char find_remotedisplaystr[] =
"CR    = Play Current    BS   = Delete Char\r\n"
"Down  = Next Match      Up   = Previous Match\r\n"
"A-Z   = Char to match   *    = Wildcard\r\n"
"Left  = Exit Find Mode"
;

char radio_remotedisplaystr[] =
"Right = Tune up         Left = Tune down\r\n"
"Down  = Next Preset     Up   = Previous Preset\r\n"
"[]    = Volume          M    = Mute\r\n"
"R     = Record          BS   = Exit Radio Mode\r\n"
"\r\n"
"-+F/<>TCN0123456789   Same as in Play Mode\r\n"
;

char record_remotedisplaystr[] =
"[]  = Volume\r\n"
"M   = Mute\r\n"
"R   = Stop Recording\r\n"
"W   = Switch to uncompressed recording\r\n"
;

int sockets[MAX_PORTS];
int num_sockets = 0;

// cursor control keys consist of three characters
// Yuck-o.

int ckey_decode_states[MAX_PORTS];

int listen_socket;

init_remoteif()
{
   struct sockaddr_in s_addr;
   listen_socket = socket(PF_INET,SOCK_STREAM,0);
   if(listen_socket<0) {
      perror("socket");
      exit(999);
   }
   s_addr.sin_family = AF_INET;
   s_addr.sin_port = htons(SERVER_PORT);
   s_addr.sin_addr.s_addr = htonl(INADDR_ANY);

   if(bind(listen_socket,&s_addr,sizeof(s_addr)) < 0) {
      fprintf(stderr,"bind port %d: ",SERVER_PORT);
      perror("");
      exit(999);
   }

   // Make the server socket nonblocking, otherwise the program
   // can block in accept() if a connection is dropped before
   // we can accept it.

   if(fcntl(listen_socket,F_SETFL,O_NONBLOCK)<0) {
      perror("socket fcntl");
      exit(999);
   }

   if(listen(listen_socket,5)<0) {
      perror("listen");
      exit(999);
   }

   // Writing to a socket whose connection is closed results in
   // a SIGPIPE, if we don't handle it the program will be killed.

   signal(SIGPIPE,SIG_IGN);
}

nuke_connection(int which)
{
   int j;
   close(sockets[which]);
   for(j=which;j<(num_sockets-1);j++) {
      sockets[j]=sockets[j+1];
      ckey_decode_states[j] = ckey_decode_states[j+1];
   }
   num_sockets--;
}

update_remote_displays(int menu,char *what,int which)
{
   int i;
   static char *lastmodestr = 0;
   char *modestr;

   if(recording) {
      modestr = record_remotedisplaystr;
   } else if(find_mode) {
      modestr = find_remotedisplaystr;
   } else if(radio_mode) {
      modestr = radio_remotedisplaystr;
   } else {
      modestr = play_remotedisplaystr;
   }

   if(modestr != lastmodestr) menu = 1;
   lastmodestr = modestr;

   for(i=0;i<num_sockets;i++) {
      char buf[100];
      int n;
      if(which>=0 && which!=sockets[i]) continue;

      if(menu) {
         write(sockets[i],common_remotedisplaystr,strlen(common_remotedisplaystr));
         write(sockets[i],modestr,strlen(modestr));
      }
      strcpy(buf,"\033[H\n| ");
      strncat(buf,what,40);
      strcat(buf,"\r\n| ");
      strncat(buf,what+40,40);
      strcat(buf,"\r\n\n");

      // Simply do a blocking write to all the connected
      // sockets.  Yes, this means if one of them gets backlogged
      // the whole program will block, it won't even accept new
      // connections until the backlog clears.

      // Also don't bother to check if all the data actually got written.

      n = write(sockets[i],buf,strlen(buf));

      // Any error at all means drop the connection (this includes
      // the EPIPE that says the other end dropped it.)

      if(n<0) nuke_connection(i);
   }
}

update_all_remote_displays(char *what)
{
   update_remote_displays(0,what,-1);
}

remoteif_do_fd_set(fd_set *readfds, int *maxfd)
{
   int i;
   for(i=0;i<num_sockets;i++) {
      FD_SET(sockets[i],readfds);
      if(sockets[i]>*maxfd) *maxfd=sockets[i];
   }
   FD_SET(listen_socket,readfds); 
   if(listen_socket>*maxfd) *maxfd = listen_socket;
}

int remoteif_do_fd_check(fd_set *readfds)
{
   int i;
   int remote_code = 0;
   for(i=0;i<num_sockets;i++) {

      // Stop getting characters as soon as a remote control code
      // is determined.  Otherwise simultaneous keypresses would be lost.

      if(remote_code) break;

      if(FD_ISSET(sockets[i],readfds)) {
         char c;
         struct kmap *kp;
         int r = read(sockets[i],&c,1);

         // any kind of error closes this connection

         if(r<=0) {
            nuke_connection(i);
            continue;
         }
         //printf("got %c from socket %d\n",c,i);
         if(c==12) {
            update_remote_displays(1,lcd_contents,sockets[i]);
            break;
         }
         if(c==27) {
            ckey_decode_states[i] = 1;
            continue;
         }
         if(ckey_decode_states[i] == 1) {
            if(c=='[') {
               ckey_decode_states[i] = 2;
               continue;
            } else {
               ckey_decode_states[i] = 0;
            }
         }
         if(ckey_decode_states[i] == 2) {
            switch(c) {
               case 'A':
                  remote_code = UP_CODE;
                  break;
               case 'B':
                  remote_code = DOWN_CODE;
                  break;
               case 'C':
                  remote_code = RIGHT_CODE;
                  break;
               case 'D':
                  remote_code = LEFT_CODE;
                  break;
            }
            ckey_decode_states[i] = 0;

            // Once we've seen ESC [, eat the next character whether
            // known or not.  This causes other magic keys (home, end etc)
            // to be ignored cleanly.

            continue;
         }
         c = toupper(c);
         if(!recording && find_mode && c>='A' && c<='Z') {
            remote_code = alpha_codes[c-'A'];
            break;
         }

         if(recording) kp = &record_key_mappings[0];
         else if(radio_mode) kp = &radio_key_mappings[0];
         else kp = &play_key_mappings[0];

         while(kp->c) {
            if(c==kp->c) {
               remote_code = kp->code;
               break;
            }
            kp++;
         }
      }
   }

   // HACK! HACK! HACK!  In find mode, swap backspace and cursor left

   if(find_mode) {
      if(remote_code == STOP_CODE) remote_code = LEFT_CODE;
      else if(remote_code == LEFT_CODE) remote_code = STOP_CODE;
   }


   // Handle a new connection

   if(FD_ISSET(listen_socket,readfds)) {
      struct sockaddr_in c_addr;
      int c_addr_size;
      int s = accept(listen_socket,&c_addr,&c_addr_size);

      // Drop it immediately if we have the maximum number of
      // connections already.

      if(num_sockets<MAX_PORTS) {
         ckey_decode_states[num_sockets] = 0;
         sockets[num_sockets++] = s;
         update_remote_displays(1,lcd_contents,s);
      } else {
         close(s);
      }
   }

   return remote_code;
}
