#include "jukebox.h"

#define MAX_ENTRIES 100

struct timer_entry {
    int weekday;
    int freq;
    int hour;
    int minute;
    int duration;
} timer_table[MAX_ENTRIES];

int num_timers = 0;

unsigned long timer_wakeup_time = 0;
int timer_duration;
int timer_freq;

char buf[1024];

char *weekdays[8] = {
   "daily","sun","mon","tue","wed","thu","fri","sat"
};

init_timer()
{
   FILE *fp;

   fp = fopen("timer.settings","r");
   if(!fp) {
      fprintf(stderr,"can't read timer settings, timer disabled\n");
      return;
   }
   while(fgets(buf,sizeof(buf),fp)) {
      char day[100];
      float freq;
      int hour;
      int minute;
      int duration;
      int i;
      if(*buf == '#') continue;
      if(5!=sscanf(buf,"%s %f %d:%d %d",day,&freq,&hour,&minute,&duration)) {
         fprintf(stderr,"Parse error: %s",buf);
         continue;
      }
      for(i=0;i<8;i++) if(!strncasecmp(weekdays[i],day,strlen(weekdays[i]))) break;
      if(i==8) {
         fprintf(stderr,"Parse error: %s",buf);
         continue;
      }
      timer_table[num_timers].weekday = i;
      timer_table[num_timers].freq = (int) (10*freq+.5);
      timer_table[num_timers].hour = hour;
      timer_table[num_timers].minute = minute;
      timer_table[num_timers].duration = duration;
      printf("timer%2d: %-6s%4d.%d  %02d:%02d  %d minutes\n",
         num_timers,weekdays[i],timer_table[num_timers].freq/10,
         timer_table[num_timers].freq%10,hour,minute,duration);
      num_timers++;
   }
   fclose(fp);
   schedule_next_timer_event();
}

schedule_next_timer_event()
{
   time_t t = time(0);
   time_t new_time;
   int i;
   timer_wakeup_time = 0;
   //printf("current time %s",ctime(&t));

   if(!have_radio) {
      printf("no timer event scheduled because no radio hardware is available\n");
      return;
   }

   for(i=0;i<num_timers;i++) {
      struct tm *tm = localtime(&t);
      tm->tm_hour = timer_table[i].hour;
      tm->tm_min  = timer_table[i].minute;
      tm->tm_sec  = 0;
      if(timer_table[i].weekday) {
         while(tm->tm_wday != (timer_table[i].weekday-1)) {
            tm->tm_wday = (tm->tm_wday+1) % 7;
            tm->tm_mday++;  // note the manpage says it's OK to go past
                         // the end of the month; mktime will fix it.
         }
      }
      for(;;) {
         new_time = mktime(tm);
         //printf("timer %d try time: %s",i,ctime(&new_time));
         if(new_time > t) break;
         tm->tm_mday += timer_table[i].weekday ? 7 : 1;
      }
      if(!timer_wakeup_time || new_time < timer_wakeup_time) {
         timer_wakeup_time = new_time;
         timer_duration = timer_table[i].duration;
         timer_freq = timer_table[i].freq;
         //printf("wakeup time is now %s",ctime(&timer_wakeup_time));
      }
   }
   printf("next timer event: %s  on %d.%dMHz for %d minutes\n",
      ctime(&timer_wakeup_time),timer_freq/10,timer_freq%10,timer_duration);
}
