#include "jukebox.h"

#define VOICE_DIR "voice"

#define BUFSIZE 1024

int audio_fd = -1;

char *filenames[] = {
   "zero",
   "one",
   "two",
   "three",
   "four",
   "five",
   "six",
   "seven",
   "eight",
   "nine",
   "ten",
   "eleven",
   "twelve",
   "thirteen",
   "fourteen",
   "fifteen",
   "sixteen",
   "seventeen",
   "eighteen",
   "nineteen",
   "twenty",
   "thirty",
   "fourty",
   "fifty",
   "sixty",
   "seventy",
   "eighty",
   "ninety",
};

char *numfilename(digit)
{
   return filenames[digit];
}

say_continuemode()
{
   open_audio();
   switch(continue_mode) {
      case cm_single:
         speak_file("single");
         speak_file("track");
         speak_file("mode");
         break;
      case cm_trackloop:
         speak_file("track");
         speak_file("loop");
         speak_file("mode");
         break;
      case cm_album: 
         speak_file("single");
         speak_file("album");
         speak_file("mode");
         break;
      case cm_albumloop:
         speak_file("album");
         speak_file("loop");
         speak_file("mode");
         break;
      case cm_continuous:
         speak_file("infinite");
         speak_pause(100);
         speak_file("play");
         speak_file("mode");
         break;
      case cm_faveloop:
         speak_file("favourites");
         speak_file("loop");
         speak_file("mode");
         break;
   }
   close_audio();
}

speak_file_with_open(char *what)
{
   open_audio();
   speak_file(what);
   close_audio();
}

unsigned char buf[BUFSIZE];

speak_file(char *what)
{
   FILE *fp;
   int bp;
   int i;
   short *sp;
   char namebuf[1000];
   strcpy(namebuf,VOICE_DIR);
   strcat(namebuf,"/");
   strcat(namebuf,what);
   fp = fopen(namebuf,"r");
   if(!fp) {
      fprintf(stderr,"Could not open '%s'\n",namebuf);
      exit(999);
   }
   bp = 0;
   do {
      while(bp<BUFSIZE) {
         int r = fread(buf+bp,1,BUFSIZE-bp,fp);
         if(r<=0) break;
         bp += r;
      }
      sp = (short *)buf;
      for(i=bp/2;i--;) {
         *sp++ <<= 2;
      }
      write(audio_fd,buf,bp);
      bp = 0;
   } while(!feof(fp));
   fclose(fp);
}

speak_pause(int l)
{
   short buf[11];
   int i;
   for(i=0;i<11;i++) buf[i]=0;
   while(l--) {
      write(audio_fd,buf,sizeof(buf));
   }
}

say_time()
{
   time_t tt = time(0);
   struct tm *t = localtime(&tt);
   char *ampm = "am";
   int hour = t->tm_hour;
   int minute = t->tm_min;
   open_audio();
   if(hour==0) {
      hour = 12;
      if(minute==0) ampm = "pm";
   } else if(hour==12) {
      if(minute>0) ampm = "pm";
   } else if(hour>12) {
      hour -= 12;
      ampm = "pm";
   }
   saynum(hour);
   if(t->tm_min) {
      speak_pause(200);
      if(t->tm_min<10) speak_file("oh");
      saynum(t->tm_min);
   }
   speak_pause(100);
   speak_file(ampm);
   close_audio();
}

saynum(int n)
{
  if(n>999999) {
     int millions = n/1000000;
     saynum(millions);
     speak_file("million");
     n -= 1000000*millions;
     if(!n) return;
  }
  if(n>1999) {
     int thousands = n/1000;
     saynum(thousands);
     speak_file("thousand");
     n -= 1000*thousands;
     if(!n) return;
  } 
  if(n>99) {
     int hundreds = n/100;
     saynum(hundreds);
     speak_file("hundred");
     n -= 100*hundreds;
     if(!n) return;
  }
  if(n>20) {
     int tens = n/10;
     speak_file(filenames[18+tens]); 
     n -= 10*tens;
     if(!n) return;
  }
  speak_file(filenames[n]);
}

open_audio()
{
  int r;
  int val;

  if(audio_fd >= 0) return;

  audio_fd = open(DSPDEV,O_WRONLY);  
  if(audio_fd<0) {
     fprintf(stderr,"Can't open audio driver.\n");
     exit(999);
  }
  r = ioctl(audio_fd,SNDCTL_DSP_RESET,NULL);
  if(r>=0) {
     val = 16;
     r = ioctl(audio_fd,SNDCTL_DSP_SAMPLESIZE,&val);
  }
  if(r>=0) {
     val = AFMT_S16_LE;
     r = ioctl(audio_fd,SNDCTL_DSP_SETFMT,&val);
  }
  if(r>=0) {
     val = 0;
     r = ioctl(audio_fd,SNDCTL_DSP_STEREO,&val);
  }
  if(r>=0) {
     val = 11025;
     r = ioctl(audio_fd,SNDCTL_DSP_SPEED,&val);
  }
  if(r<0) {
     fprintf(stderr,"Sound device did not accept settings.\n");
     exit(999);
  }
}

close_audio()
{
   if(audio_fd<0) return;
   close(audio_fd);
   audio_fd = -1;
}
